USE [DBA]
GO

/****** Object:  StoredProcedure [dbo].[GetRingBufferDetails]    Script Date: 01/02/2010 19:29:27 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO



/******************************************************************************
**    Auth: Bill Wunder
**    Date: Nov 4, 2009
**
**    Desc: parse some details from the ring buffer DMV
**
*******************************************************************************
**    Change History
*******************************************************************************
**    Date           Author             Description of Change
**              
*******************************************************************************/
CREATE PROC [dbo].[GetRingBufferDetails]
AS 
BEGIN

	SET NOCOUNT ON

	/*
	http://blogs.msdn.com/psssql/archive/2008/03/24/how-it-works-sql-server-2005-sp2-security-ring-buffer-ring-buffer-security-error.aspx
	SQL Server 2005 SP2 added new ring buffer entries (sys.dm_os_ring_buffers) for various security errors.   The reason the ring buffer entries are added was to provide the DBA with more details as to why a client is receiving a failed login or other such error.

		* The ring buffer entries are protected because they require view system state privileges to enumerate. 
		* The information contained in the ring buffer can be used to help understand the source of a failure.   
		* The failure information is not reported in the error message to the client to prevent information disclosure that would allow an attack.  For example if you report to the client that the password was incorrect an attacker knows they can start trying a series of passwords

	ring buffer timestamp to datetime
	this is claimed to work but doesnt
	http://social.msdn.microsoft.com/Forums/en-US/sqldatabaseengine/thread/32217f4f-bd3c-423a-b02d-ab5bba46f2c2
	select DATEADD (ms, -1 * ((i.cpu_ticks / i.cpu_ticks_in_ms) - r.timestamp), GETDATE())  as record_time
	FROM sys.dm_os_ring_buffers r
	CROSS JOIN sys.dm_os_sys_info i
	this gives a date for some rows - no clue if valid
	select dateadd (ms, r.[timestamp] - i.ms_ticks, getdate()) as record_time
	FROM sys.dm_os_ring_buffers r
	CROSS JOIN sys.dm_os_sys_info i
	*/    

	declare @summary table
		( [ring_buffer_type] NVARCHAR(128)
		, [type_count] INT )
		INSERT @summary ([ring_buffer_type]
		, [type_count])
	SELECT [ring_buffer_type]
		, COUNT(*) AS [type_count]
	FROM sys.dm_os_ring_buffers r
	CROSS JOIN sys.dm_os_sys_info i
	GROUP BY [ring_buffer_type]

	SELECT * FROM @summary

	IF EXISTS (SELECT * 
			   FROM @summary 
			   WHERE [ring_buffer_type] = 'RING_BUFFER_CLRAPPDOMAIN')
		SELECT COUNT(*) AS [count]
			, 'RING_BUFFER_CLRAPPDOMAIN' AS [Type]
			, Action
		FROM (	SELECT RingBuffer.[Record].value('Action[1]', 'varchar(100)') AS [Action] 
				FROM (SELECT CAST([Record] AS [XML]) AS [TargetData] 
					  FROM sys.dm_os_ring_buffers
					  WHERE [ring_buffer_type] = 'RING_BUFFER_CLRAPPDOMAIN') AS Data
				CROSS APPLY [TargetData].nodes('//Record') AS RingBuffer([Record])) derived
		GROUP BY [Action]
		ORDER BY [Action]

	--http://blogs.msdn.com/slavao/archive/2005/02/19/376714.aspx
	IF EXISTS (SELECT * 
			   FROM @summary 
			   WHERE [ring_buffer_type] = 'RING_BUFFER_RESOURCE_MONITOR')
	SELECT 'RING_BUFFER_RESOURCE_MONITOR' AS [Type], RingBuffer.[Record].query('.')--RingBuffer.Record.value('Error[1]', 'int') as error
				FROM (SELECT CAST([Record] AS [XML]) AS [TargetData] 
					  FROM sys.dm_os_ring_buffers
					  WHERE [ring_buffer_type] = 'RING_BUFFER_RESOURCE_MONITOR') AS Data
				CROSS APPLY [TargetData].nodes('//Record') AS RingBuffer([Record])

	/*
	SELECT RingBuffer.Record.query('.') as [Entry]
				FROM (SELECT CAST(Record AS XML) AS TargetData 
					  FROM sys.dm_os_ring_buffers
					  WHERE ring_buffer_type = 'RING_BUFFER_SCHEDULER_MONITOR') AS Data
				CROSS APPLY TargetData.nodes('//Record/SchedulerMonitorEvent') AS RingBuffer(Record)
	*/			
				

	IF EXISTS (SELECT * 
			   FROM @summary 
			   WHERE ring_buffer_type = 'RING_BUFFER_MEMORY_BROKER')
		SELECT COUNT(*) AS [count]
			, 'RING_BUFFER_MEMORY_BROKER' AS [Type]
			, [Pool]
			, [Broker]
			, [Notification]
		FROM (	SELECT RingBuffer.Record.value('Pool[1]', 'int') AS [Pool]
					, RingBuffer.Record.value('Broker[1]', 'varchar(100)') AS [Broker]
					, RingBuffer.Record.value('Notification[1]', 'varchar(100)') AS [Notification]
				FROM (SELECT CAST(Record AS XML) AS TargetData 
					  FROM sys.dm_os_ring_buffers
					  WHERE ring_buffer_type = 'RING_BUFFER_MEMORY_BROKER') AS Data
				CROSS APPLY TargetData.nodes('//Record/MemoryBroker') AS RingBuffer(Record)) derived
		GROUP BY [Pool]
			, [Broker]
			, [Notification]
		ORDER BY [Pool]
			, [Broker]
			, [Notification]

	IF EXISTS (SELECT * 
			   FROM @summary 
			   WHERE ring_buffer_type = 'RING_BUFFER_SECURITY_ERROR')
		SELECT COUNT(*) AS [count]
			, 'RING_BUFFER_SECURITY_ERROR' AS [Type]
			, [APIName]
			, [CallingAPIName]
			, [ErrorCode]
		FROM (	SELECT RingBuffer.Record.value('APIName[1]', 'varchar(128)') as [APIName]
					, RingBuffer.Record.value('CallingAPIName[1]', 'varchar(128)') as [CallingAPIName]
					, RingBuffer.Record.value('ErrorCode[1]', 'varchar(128)') as [ErrorCode]
				FROM (SELECT CAST(Record AS XML) AS TargetData 
					  FROM sys.dm_os_ring_buffers
					  WHERE ring_buffer_type = 'RING_BUFFER_SECURITY_ERROR') AS Data
				CROSS APPLY TargetData.nodes('//Record/Error') AS RingBuffer(Record)) derived
		GROUP BY [APIName], [CallingAPIName], [ErrorCode]
		ORDER BY [APIName], [CallingAPIName], [ErrorCode]

	IF EXISTS (SELECT * 
			   FROM @summary 
			   WHERE ring_buffer_type = 'RING_BUFFER_XE_BUFFER_STATE')
		SELECT COUNT(*) AS [count]
			, 'RING_BUFFER_XE_BUFFER_STATE' AS [Type]
			, [NewState]
		FROM (	SELECT RingBuffer.Record.value('NewState[1]', 'varchar(100)') as [NewState]
				FROM (SELECT CAST(Record AS XML) AS TargetData 
					  FROM sys.dm_os_ring_buffers
					  WHERE ring_buffer_type = 'RING_BUFFER_XE_BUFFER_STATE') AS Data
				CROSS APPLY TargetData.nodes('//Record/XE_BufferStateRecord ') AS RingBuffer(Record)) derived
		GROUP BY [NewState]
		ORDER BY [NewState]

	IF EXISTS (SELECT * 
			   FROM @summary 
			   WHERE ring_buffer_type = 'RING_BUFFER_SCHEDULER')
		SELECT COUNT(*) AS [count]
			, 'RING_BUFFER_SCHEDULER' AS [Type]
			, [Action]
		FROM (	SELECT RingBuffer.Record.value('Action[1]', 'varchar(128)') as [Action]
				FROM (SELECT CAST(Record AS XML) AS TargetData 
					  FROM sys.dm_os_ring_buffers
					  WHERE ring_buffer_type = 'RING_BUFFER_SCHEDULER') AS Data
				CROSS APPLY TargetData.nodes('//Record/Scheduler') AS RingBuffer(Record)) derived
		GROUP BY [Action]
		ORDER BY [Action]

	IF EXISTS (SELECT * 
			   FROM @summary 
			   WHERE ring_buffer_type = 'RING_BUFFER_EXCEPTION')
        SELECT derived2.[count]
			, derived2.[Type]
			, derived2.[error]
			, m.text AS [sys.messages text for error] 
		FROM(	SELECT COUNT(*) AS [count]
					, 'RING_BUFFER_EXCEPTION' AS [Type]
					, [error]
				FROM (	SELECT RingBuffer.Record.value('Error[1]', 'int') as error
						FROM (SELECT CAST(Record AS XML) AS TargetData 
							  FROM sys.dm_os_ring_buffers
							  WHERE ring_buffer_type = 'RING_BUFFER_EXCEPTION') AS Data
						CROSS APPLY TargetData.nodes('//Record/Exception') AS RingBuffer(Record)) derived1
				GROUP BY [error])derived2
		LEFT JOIN sys.messages m
		ON derived2.[error] = m.message_id 
		AND m.[language_id] = SERVERPROPERTY('LCID')
		ORDER BY [error] 

	IF EXISTS (SELECT * 
			   FROM @summary 
			   WHERE ring_buffer_type = 'RING_BUFFER_CONNECTIVITY')
		SELECT COUNT(*) AS [count]
			, 'RING_BUFFER_CONNECTIVITY' AS [Type]
			, [RecordType]
		FROM (	SELECT RingBuffer.Record.value('RecordType[1]', 'varchar(100)') as [RecordType]
				FROM (SELECT CAST(Record AS XML) AS TargetData 
					  FROM sys.dm_os_ring_buffers
					  WHERE ring_buffer_type = 'RING_BUFFER_CONNECTIVITY') AS Data
				CROSS APPLY TargetData.nodes('//Record/ConnectivityTraceRecord') AS RingBuffer(Record)) derived
		GROUP BY [RecordType]
		ORDER BY [RecordType]

	IF EXISTS (SELECT * 
			   FROM @summary 
			   WHERE ring_buffer_type = 'RING_BUFFER_XE_LOG')
		SELECT COUNT(*) AS [count]
			, 'RING_BUFFER_XE_LOG' AS [Type]
			, [XE_LogRecord]
		FROM (	SELECT RingBuffer.Record.value('(@message)[1]', 'varchar(max)') AS [XE_LogRecord]
				FROM (SELECT CAST(Record AS XML) AS TargetData 
					  FROM sys.dm_os_ring_buffers
					  WHERE ring_buffer_type = 'RING_BUFFER_XE_LOG') AS Data
				CROSS APPLY TargetData.nodes('//Record/XE_LogRecord') AS RingBuffer(Record)) derived
		GROUP BY [XE_LogRecord]
		ORDER BY [XE_LogRecord]
END


GO

