USE [DBA]
GO

/****** Object:  StoredProcedure [dbo].[GetXESystemHealthRecentDetails]    Script Date: 01/02/2010 19:30:32 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

/******************************************************************************
**    Auth: Bill Wunder
**    Date: Nov 4, 2009
**
**    Desc: get details from system health ring buffer content
**
*******************************************************************************
**    Change History
*******************************************************************************
**    Date           Author             Description of Change
**              
*******************************************************************************/
CREATE PROC [dbo].[GetXESystemHealthRecentDetails]
( @DeadLocksToShow AS [INT] = 20 
, @DaysToShow [INT] = 2)
AS
BEGIN
	DECLARE @ring AS [XML]
	DECLARE @summary TABLE 
		( EventName [VARCHAR] (100)
		, EventDate [DATE]
		, EventCount [INT]
		, LastOccurence [DATETIME]);
		
	-- seems to run much faster if local variable used rather than to directly query the DMVs and retrun results 
	SELECT @ring = CAST(xet.[target_data] AS [XML])
	FROM sys.dm_xe_session_targets xet
	INNER JOIN sys.dm_xe_sessions xe
	ON (xe.[address] = xet.[event_session_address])
	WHERE xe.[name] = 'system_health';

	INSERT @summary 
		( [EventName]
		, [EventDate]
		, [EventCount]
		, [LastOccurence])
	SELECT [EventName]
		, CAST([EventTime] AS DATE) AS [Date]
		, COUNT(*) AS [Count]
		, MAX([EventTime]) AS [LastOccurence]
	FROM (	SELECT XEventData.[XEvent].value('(@name)[1]', 'varchar (100)') + ' ' 
				 + CASE WHEN XEventData.[XEvent].value('(@name)[1]', 'varchar (100)') = 'error_reported'
						THEN XEventData.[XEvent].value('(data/value)[1]', 'varchar(100)')
						ELSE '' END  AS [EventName]
				 , XEventData.[XEvent].value('(@timestamp)', 'DATETIME') AS [EventTime]
			FROM (	SELECT @ring AS [TargetData]) AS Data
			CROSS APPLY [TargetData].nodes ('//RingBufferTarget/event') AS XEventData ([XEvent]) ) derived
	GROUP BY [EventName], CAST([EventTime] AS [DATE])
	ORDER BY [EventName], [DATE] DESC;

	IF EXISTS (	SELECT * 
	            FROM @summary 
	            WHERE [EventName] = 'error_reported 17830' 
	            AND [EventDate] > GETDATE() - @DaysToShow)
		SELECT  COUNT(*) AS [Count]
			  , [XEventDay] AS [Date]  
			  , SUBSTRING( [XMessage]
		  				 , CHARINDEX('[CLIENT:', [XMessage]) + 8
						 , LEN([XMessage]) - CHARINDEX('[CLIENT:', [XMessage]) - 8) AS [SourceIP of 'Network error code 0x2746']
		FROM (	SELECT XEventData.[XEvent].value('data[@name=''message''][1]/value[1]', 'varchar(max)') AS [XMessage]
					 , CAST(XEventData.[XEvent].value('(@timestamp)', 'DATETIME') AS DATE) AS [XEventDay]  	
				FROM (	SELECT @ring AS [TargetData]) AS XData
				CROSS APPLY [TargetData].nodes ('//RingBufferTarget/event') AS [XEventData] ([XEvent]) 
				WHERE XEventData.[XEvent].value('@name', 'varchar(4000)') = 'error_reported'
				AND XEventData.[XEvent].value('data[@name = ''error''][1]/value[1]', 'varchar(100)') =  17830
				AND XEventData.[XEvent].value('(@timestamp)[1]', 'DATETIME') > CAST(CURRENT_TIMESTAMP - @DaysToShow AS DATE)) derived
		GROUP BY SUBSTRING( [XMessage]
						  , CHARINDEX('[CLIENT:', [XMessage]) + 8
						  , LEN([XMessage]) - CHARINDEX('[CLIENT:', [XMessage]) - 8)
				, [XEventDay]		  
		ORDER BY [SourceIP of 'Network error code 0x2746'], [Date];
						 
	IF EXISTS (	SELECT * from @summary 
	            WHERE [EventName] LIKE 'error_reported%'
				AND [EventName] <> 'error_reported 17830'
				AND [EventDate] > GETDATE() - @DaysToShow)
		SELECT  COUNT(*) AS [Count]
			  , [XEventDay] AS [Date]  
			  , [XError] AS [Error]
			  , [XMessage] AS [Message]
		FROM (	SELECT XEventData.[XEvent].value('data[@name = ''error''][1]/value[1]', 'varchar(100)') AS [XError]
					 , XEventData.[XEvent].value('data[@name=''message''][1]/value[1]', 'varchar(max)') AS [XMessage]
					 , CAST(XEventData.[XEvent].value('(@timestamp)', 'DATETIME') AS DATE) AS [XEventDay]  	
				FROM (	SELECT @ring AS [TargetData]) AS XData
				CROSS APPLY [TargetData].nodes ('//RingBufferTarget/event') AS XEventData ([XEvent]) 
				WHERE XEventData.[XEvent].value('@name', 'varchar(4000)') = 'error_reported'
				AND XEventData.[XEvent].value('data[@name = ''error''][1]/value[1]', 'varchar(100)') <>  17830
				AND XEventData.[XEvent].value('(@timestamp)[1]', 'DATETIME') > CAST(CURRENT_TIMESTAMP - @DaysToShow AS DATE)) derived
		GROUP BY [XError]
			   , [XMessage]
			   , [XEventDay]		  
		ORDER BY [Date], [Error], [Message];
			   
	IF EXISTS (	SELECT * 
	            FROM @summary 
	            WHERE [EventName] = 'wait_info' 
	            AND [EventDate] > GETDATE() - @DaysToShow)
		BEGIN
			SELECT XEventData.[XEvent].value('(data[@name = ''wait_type''])[1]/text[1]','varchar(100)') AS [Wait type]
				, XEventData.[XEvent].value('@timestamp','DateTime') AS [DateTime]
				, XEventData.[XEvent].value('(data[@name = ''duration''])[1]','bigint')/100000.0 AS [ElapsedSecs]
				, XEventData.[XEvent].value('(action[@name = ''sql_text''])[1]','varchar(max)') AS [SQL Text]
				, XEventData.[XEvent].query('.') AS [wait_info - XML (see sys.dm_os_wait_stats in BOL)]
			FROM (SELECT @ring AS [TargetData]) AS Data
			CROSS APPLY [TargetData].nodes ('//RingBufferTarget/event') AS XEventData ([XEvent])
			WHERE XEventData.[XEvent].value('@name', 'varchar(4000)') = 'wait_info'
			AND XEventData.[XEvent].value('(@timestamp)[1]', 'DATETIME') > CAST(CURRENT_TIMESTAMP - @DaysToShow AS DATE);

			WITH Waits AS
			(SELECT [wait_type], [wait_time_ms] / 1000.0 AS [wait_time_s],
				100.0 * [wait_time_ms] / SUM([wait_time_ms]) OVER() AS [pct],
				ROW_NUMBER() OVER(ORDER BY [wait_time_ms] DESC) AS [rn]
			 FROM sys.dm_os_wait_stats
			 WHERE [wait_type] NOT IN('SLEEP_TASK', 'BROKER_TASK_STOP', 
			  'SQLTRACE_BUFFER_FLUSH', 'CLR_AUTO_EVENT', 'CLR_MANUAL_EVENT',
			  'LAZYWRITER_SLEEP')) -- filter out some irrelevant waits
			SELECT W1.[wait_type], 
			  CAST(W1.[wait_time_s] AS DECIMAL(12, 2)) AS [wait_time_s],
			  CAST(W1.[pct] AS DECIMAL(12, 2)) AS [pct],
			  CAST(SUM(W2.[pct]) AS DECIMAL(12, 2)) AS [running_pct]
			FROM Waits AS W1
			INNER JOIN Waits AS W2
			ON W2.[rn] <= W1.[rn]
			GROUP BY W1.[rn]
				, W1.[wait_type]
				, W1.[wait_time_s]
				, W1.[pct]
			HAVING SUM(W2.[pct]) - W1.[pct] < 95; -- percentage threshold for waits

		END   

	IF EXISTS (	SELECT * from @summary where EventName = 'wait_info_external' and EventDate > GETDATE() - @DaysToShow)
		SELECT XEventData.XEvent.value('(data[@name = ''wait_type''])[1]/text[1]','varchar(100)') AS [Wait type]
			, XEventData.XEvent.value('@timestamp','DateTime') AS [DateTime]
			, XEventData.XEvent.value('(data[@name = ''duration''])[1]','bigint') AS [duration]
			, XEventData.XEvent.value('(action[@name = ''sql_text''])[1]','varchar(max)') AS [SQL Text]
			, XEventData.XEvent.query('.') AS [wait_info - XML (see sys.dm_os_wait_stats in BOL)]
		FROM (SELECT @ring AS TargetData) AS Data
		CROSS APPLY TargetData.nodes ('//RingBufferTarget/event') AS XEventData (XEvent)
		WHERE XEventData.XEvent.value('@name', 'varchar(4000)') = 'wait_info_external'
		AND XEventData.XEvent.value('(@timestamp)[1]', 'DATETIME') > CAST(CURRENT_TIMESTAMP - @DaysToShow AS DATE);

	IF EXISTS ( SELECT * FROM @summary WHERE EventName = 'xml_deadlock_report' and EventDate > GETDATE() - @DaysToShow)
		-- if more than one victim, only the last victimProcess tag is closed 
		-- this will work for up to 4 victims - so far we have only seen three victims	
		SELECT CAST(CASE WHEN CHARINDEX( CHAR(34) + CHAR(62)	
	 								   , SUBSTRING(DeadlockGraph, LastTagClose, LEN(DeadlockGraph) - LastTagClose)
									   , CHARINDEX( '<victimProcess id="process'
												  , SUBSTRING( DeadlockGraph
															 , LastTagClose
															 , LEN(DeadlockGraph) - LastTagClose))) < CHARINDEX( '<victimProcess id="process'
																											   , SUBSTRING( DeadlockGraph
																														  , LastTagClose
																														  , LEN(DeadlockGraph) - LastTagClose)) + 26 + 16
						 THEN STUFF(DeadlockGraph, CHARINDEX(CHAR(34) + CHAR(62),DeadlockGraph, CHARINDEX('<victimProcess id="process',DeadlockGraph)), 2, CHAR(34) + CHAR(47) + CHAR(62))
						 ELSE DeadlockGraph
						 END AS XML) AS [Deadlocks]
			  , [DeadlockDateTime]    
		FROM (	SELECT CASE WHEN CHARINDEX( CHAR(34) + CHAR(62)
										  , SUBSTRING(DeadlockGraph, LastTagClose, LEN(DeadlockGraph) - LastTagClose)
										  , CHARINDEX( '<victimProcess id="process'
													  , SUBSTRING( DeadlockGraph
																 , LastTagClose
																 , LEN(DeadlockGraph) - LastTagClose))) < CHARINDEX( '<victimProcess id="process'
																												   , SUBSTRING( DeadlockGraph
																															  , LastTagClose
																															  , LEN(DeadlockGraph) - LastTagClose)) + 26 + 16
							THEN STUFF(DeadlockGraph, CHARINDEX(CHAR(34) + CHAR(62),DeadlockGraph, CHARINDEX('<victimProcess id="process',DeadlockGraph)), 2, CHAR(34) + CHAR(47) + CHAR(62))
							ELSE DeadlockGraph
							END AS DeadlockGraph
						, CHARINDEX(CHAR(34) + CHAR(62),DeadlockGraph, CHARINDEX('<victimProcess id="process',DeadlockGraph)) + 2 AS [LastTagClose]
						, [DeadlockDateTime]
				FROM (	SELECT CASE WHEN CHARINDEX( CHAR(34) + CHAR(62)
												  , DeadlockGraph
												  , CHARINDEX('<victimProcess id="process',DeadlockGraph)) < CHARINDEX('<victimProcess id="process'
																													  , DeadlockGraph) + 26 + 16
									THEN STUFF(DeadlockGraph, CHARINDEX(CHAR(34) + CHAR(62),DeadlockGraph, CHARINDEX('<victimProcess id="process',DeadlockGraph)), 2, CHAR(34) + CHAR(47) + CHAR(62))
									ELSE DeadlockGraph
									END AS DeadlockGraph
							   , CHARINDEX(CHAR(34) + CHAR(62),DeadlockGraph, CHARINDEX('<victimProcess id="process',DeadlockGraph)) + 2 AS [LastTagClose]
							   , [DeadlockDateTime]
						FROM (	SELECT TOP(@DeadLocksToShow) 
								 CASE WHEN CHARINDEX('<victim-list/>', XEventData.XEvent.value('(data/value)[1]', 'varchar(max)')) = 0
									  THEN REPLACE(REPLACE(XEventData.XEvent.value('(data/value)[1]', 'varchar(max)')
												  , '<victim-list>'
												  , '<deadlock><victim-list>')
														  , '<process-list>'
														  , '</victim-list><process-list>')
									   ELSE REPLACE(XEventData.XEvent.value('(data/value)[1]', 'varchar(max)')
												   , '<victim-list/>'
												   , '<deadlock><victim-list/>') END as DeadlockGraph
										, XEventData.XEvent.value('(@timestamp)[1]', 'DATETIME') AS [DeadlockDateTime]
								FROM (SELECT @ring AS TargetData) AS Data
								CROSS APPLY TargetData.nodes ('//RingBufferTarget/event') AS XEventData (XEvent)
								WHERE XEventData.XEvent.value('@name', 'varchar(4000)') = 'xml_deadlock_report'
								AND XEventData.XEvent.value('(@timestamp)[1]', 'DATETIME') > CAST(CURRENT_TIMESTAMP - @DaysToShow AS DATE)
								ORDER BY [DeadlockDateTime] DESC) derived1 
					) derived2 
			) derived3;
			
			--scheduler_monitor_non_yielding_ring_buffer_recorded 
	IF EXISTS (	SELECT * 
	            FROM @summary 
	            WHERE EventName NOT IN ( 'error_reported'
	                                   , 'wait_info'
	                                   , 'wait_info_external'
	                                   , 'xml_deadlock_report') and EventDate > GETDATE() - @DaysToShow)
		SELECT XEventData.XEvent.value('(@name)[1]', 'varchar (100)') AS [Event Name]
		    , XEventData.XEvent.value('(@timestamp)[1]', 'DATETIME') AS [Event Date]
			, XEventData.XEvent.query('.') AS [XML payload]
		FROM (SELECT @ring AS TargetData) AS Data
		CROSS APPLY TargetData.nodes ('//RingBufferTarget/event') AS XEventData (XEvent)
		WHERE XEventData.XEvent.value('@name', 'varchar(4000)') NOT IN  ( 'error_reported'
																		   , 'wait_info'
																		   , 'wait_info_external'
																		   , 'xml_deadlock_report')
		AND XEventData.XEvent.value('(@timestamp)[1]', 'DATETIME') > CAST(CURRENT_TIMESTAMP - @DaysToShow AS DATE);
END

GO

