USE [DBA]
GO

/****** Object:  StoredProcedure [dbo].[GetXESystemHealthSummary]    Script Date: 01/02/2010 19:30:49 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

/******************************************************************************
**    Auth: Bill Wunder
**    Date: Nov 4, 2009
**
**    Desc: get aggregation of system health ring buffer content
**
*******************************************************************************
**    Change History
*******************************************************************************
**    Date           Author             Description of Change
**              
*******************************************************************************/
CREATE PROC [dbo].[GetXESystemHealthSummary]
( @DaysToShow [INT] = 2)
AS
BEGIN
	DECLARE @ring AS [XML]
		
	/* this will dump the events in the target to an XML listing
	SELECT XEventData.XEvent.query('.')
	FROM (	SELECT CAST(xet.target_data AS xml) as [TargetData]
			FROM sys.dm_xe_session_targets xet
			INNER JOIN sys.dm_xe_sessions xe
			ON (xe.address = xet.event_session_address)
			WHERE xe.name = 'system_health') AS Data
	CROSS APPLY TargetData.nodes ('//RingBufferTarget/event') AS XEventData (XEvent) 
	*/

	-- seems to run much faster if local variable used rather than to directly query the DMVs and retrun results 
	SELECT @ring = CAST(xet.[target_data] AS xml)
	FROM sys.dm_xe_session_targets xet
	INNER JOIN sys.dm_xe_sessions xe
	ON (xe.[address] = xet.[event_session_address])
	WHERE xe.[name] = 'system_health';

	SELECT [EventName]
	    , [EventInfo]
  		 , CAST([EventTime] AS DATE) AS [EventDate]         
		   , COUNT(*) AS [EventCount]
		   , MIN([EventTime]) AS [FirstOccurrence]
		   , MAX([EventTime]) AS [LastOccurrence]
	FROM (	SELECT XEventData.XEvent.value('(@name)[1]', 'varchar (128)') AS [EventName]
				 , CASE WHEN XEventData.[XEvent].value('(@name)[1]', 'varchar (128)') = 'error_reported'
						THEN XEventData.[XEvent].value('(data/value)[1]', 'varchar(128)')
                        WHEN XEventData.[XEvent].value('(@name)[1]', 'varchar (128)') in ('wait_info', 'wait_info_external')
                        THEN XEventData.[XEvent].value('(data[@name = ''wait_type''])[1]/text[1]','varchar(128)')
                        WHEN XEventData.[XEvent].value('(@name)[1]', 'varchar (128)') = 'xml_deadlock_report' 
                        THEN ''  
						ELSE '' END AS [EventInfo]
				 , XEventData.[XEvent].value('(@timestamp)', 'DATETIME') AS [EventTime]
			FROM (	SELECT @ring AS TargetData) AS Data
			CROSS APPLY TargetData.nodes ('//RingBufferTarget/event') AS XEventData ([XEvent]) 
  	WHERE XEventData.[XEvent].value('(@timestamp)', 'DATETIME') > CAST(CURRENT_TIMESTAMP - @DaysToShow AS [DATE])) derived
  	GROUP BY [EventName], CAST([EventTime] AS [DATE]), [EventInfo]
  	ORDER BY [FirstOccurrence] DESC, [EventName], [EventInfo];
END


GO

